jQuery(document).ready(function($) {
    // Cache DOM elements
    const form = $('#ctrlbs-booking-form');
    const timeSelect = $('#ctrlbs-booking-time');
    const dateInput = $('#ctrlbs-booking-date');
    const serviceSelect = $('#ctrlbs-service-select');
    const successMessage = $('.ctrlbs-success-message');
    const errorMessage = $('.ctrlbs-error-message');
    let submitButton;

    // Input length constraints
    const NAME_MIN = 2;
    const NAME_MAX = 50;
    const PHONE_MIN = 7;
    const PHONE_MAX = 20;
    const EMAIL_MIN = 5;
    const EMAIL_MAX = 100;

    // Add validation message containers after inputs
    $('#ctrlbs-name').after('<div class="ctrlbs-validation-message" id="ctrlbs-name-validation"></div>');
    $('#ctrlbs-phone').after('<div class="ctrlbs-validation-message" id="ctrlbs-phone-validation"></div>');
    $('#ctrlbs-email').after('<div class="ctrlbs-validation-message" id="ctrlbs-email-validation"></div>');
    $('#ctrlbs-booking-date').after('<div class="ctrlbs-validation-message" id="ctrlbs-booking-date-validation"></div>');
    $('#ctrlbs-booking-form').prepend('<div class="ctrlbs-validation-message" id="ctrlbs-form-validation"></div>');

    // Utility function to escape HTML
    function escapeHtml(string) {
        return String(string)
            .replace(/&/g, '&amp;')
            .replace(/"/g, '&quot;')
            .replace(/'/g, '&#39;')
            .replace(/</g, '&lt;')
            .replace(/>/g, '&gt;');
    }

    // Function to display messages
    function displayMessage(type, message) {
        const messageDiv = type === 'success' ? successMessage : errorMessage;
        messageDiv.text(message).show().delay(5000).fadeOut();
    }

    // Input validation function
    function validateLength(input, min, max) {
        const value = input.val().trim();
        const length = value.length;
        const messageElement = $(`#${input.attr('id')}-validation`);
        
        if (length === 0) {
            messageElement.removeClass('error').text('');
            return true;
        }
        
        if (length < min || length > max) {
            messageElement.addClass('error').text(`Must be between ${min} and ${max} characters`);
            return false;
        }
        
        messageElement.removeClass('error').text('');
        return true;
    }

    // Load services function
    function loadServices() {
        serviceSelect.prop('disabled', true);
        $.ajax({
            url: ctrlbsAjax.ajaxurl,
            type: 'POST',
            data: {
                action: 'get_services',
                booking_nonce: ctrlbsAjax.nonce
            },
            success: function(response) {
                if (response.success && response.data.services) {
                    serviceSelect.empty();
                    serviceSelect.append(`<option value="">${escapeHtml(ctrlbsAjax.i18n.selectService)}</option>`);

                    response.data.services.forEach(function(service) {
                        const option = $('<option></option>')
                            .val(escapeHtml(service.id))
                            .text(`${escapeHtml(service.name)} - ${escapeHtml(service.price)}`);
                        serviceSelect.append(option);
                    });
                } else {
                    displayMessage('error', escapeHtml(ctrlbsAjax.i18n.errorLoading));
                }
                serviceSelect.prop('disabled', false);
            },
            error: function() {
                displayMessage('error', escapeHtml(ctrlbsAjax.i18n.errorLoading));
                console.error('Failed to load services');
                serviceSelect.prop('disabled', false);
            }
        });
    }

    // Load time slots function
    function loadTimeSlots() {
        const date = escapeHtml(dateInput.val());
        const serviceId = escapeHtml(serviceSelect.val());

        timeSelect.prop('disabled', true);
        timeSelect.empty();
        timeSelect.append(`<option value="">${escapeHtml(ctrlbsAjax.i18n.selectTime)}</option>`);

        if (!date || !serviceId) {
            return;
        }

        timeSelect.append(`<option value="" disabled>${escapeHtml('Loading...')}</option>`);

        $.ajax({
            url: ctrlbsAjax.ajaxurl,
            type: 'POST',
            data: {
                action: 'get_time_slots',
                booking_nonce: ctrlbsAjax.nonce,
                date: date,
                service_id: serviceId
            },
            success: function(response) {
                timeSelect.empty();
                timeSelect.append(`<option value="">${escapeHtml(ctrlbsAjax.i18n.selectTime)}</option>`);

                if (response.success && response.data.slots && response.data.slots.length > 0) {
                    response.data.slots.forEach(function(slot) {
                        const option = $('<option></option>')
                            .val(escapeHtml(slot))
                            .text(escapeHtml(slot));
                        timeSelect.append(option);
                    });
                    timeSelect.prop('disabled', false);
                } else {
                    timeSelect.append(`<option value="" disabled>${escapeHtml('No available slots')}</option>`);
                }
            },
            error: function() {
                timeSelect.empty();
                timeSelect.append(`<option value="">${escapeHtml(ctrlbsAjax.i18n.selectTime)}</option>`);
                timeSelect.append(`<option value="" disabled>${escapeHtml('Error loading slots')}</option>`);
                displayMessage('error', escapeHtml(ctrlbsAjax.i18n.errorLoading));
            }
        });
    }

    // Date validation function
    function validateDate() {
        const selectedDate = dateInput.val();
        const today = new Date().toISOString().split('T')[0];
        const messageElement = $('#ctrlbs-booking-date-validation');

        if (selectedDate < today) {
            dateInput.val('');
            messageElement.addClass('error').text('Please select a future date');
            return false;
        }

        const date = new Date(selectedDate);
        const dayOfWeek = date.getDay();
        const days = ['sunday', 'monday', 'tuesday', 'wednesday', 'thursday', 'friday', 'saturday'];
        const selectedDay = days[dayOfWeek];
        const businessDays = ctrlbsAjax.businessDays;

        if (!businessDays[selectedDay]) {
            dateInput.val('');
            const dayName = selectedDay.charAt(0).toUpperCase() + selectedDay.slice(1);
            messageElement.addClass('error').text(`Sorry, we are closed on ${dayName}s`);
            timeSelect.prop('disabled', true);
            timeSelect.empty().append(`<option value="">${escapeHtml(ctrlbsAjax.i18n.selectTime)}</option>`);
            return false;
        }

        messageElement.removeClass('error').text('');
        return true;
    }

    function setMaxDate() {
        const today = new Date();
        const maxDays = parseInt(ctrlbsAjax.maxAdvanceDays) || 30;
        const maxDate = new Date();
        maxDate.setDate(today.getDate() + maxDays);
        dateInput.attr('max', maxDate.toISOString().split('T')[0]);
    }

    // Rate limit countdown function
    function startRetryCountdown(seconds) {
        if (!submitButton) {
            submitButton = form.find('button[type="submit"]');
        }
        const originalText = submitButton.html();
        let timeLeft = seconds;

        submitButton.prop('disabled', true);
        
        const countdown = setInterval(() => {
            if (timeLeft <= 0) {
                submitButton.prop('disabled', false).html(originalText);
                clearInterval(countdown);
                return;
            }
            submitButton.html(`Please wait ${timeLeft}s`);
            timeLeft--;
        }, 1000);
    }

    // Add input event listeners for validation
    $('#ctrlbs-name').on('input', function() {
        validateLength($(this), NAME_MIN, NAME_MAX);
    });

    $('#ctrlbs-phone').on('input', function() {
        validateLength($(this), PHONE_MIN, PHONE_MAX);
    });

    $('#ctrlbs-email').on('input', function() {
        validateLength($(this), EMAIL_MIN, EMAIL_MAX);
    });

    // Form submission handler
    form.on('submit', function(e) {
        e.preventDefault();

        // Validate all inputs before submission
        const nameValid = validateLength($('#ctrlbs-name'), NAME_MIN, NAME_MAX);
        const phoneValid = validateLength($('#ctrlbs-phone'), PHONE_MIN, PHONE_MAX);
        const emailValid = validateLength($('#ctrlbs-email'), EMAIL_MIN, EMAIL_MAX);
        const dateValid = validateDate();

        if (!nameValid || !phoneValid || !emailValid || !dateValid) {
            return;
        }
    
        const formData = new FormData(this);
        formData.append('action', 'ctrlbs_booking_submit');
        formData.append('booking_nonce', ctrlbsAjax.nonce);
    
        $.ajax({
            url: ctrlbsAjax.ajaxurl,
            type: 'POST',
            data: formData,
            processData: false,
            contentType: false,
            beforeSend: function() {
                submitButton = form.find('button[type="submit"]');
                submitButton.prop('disabled', true);
                errorMessage.hide();
                successMessage.hide();
            },
            success: function(response) {
                if (response.success) {
                    displayMessage('success', escapeHtml(response.data.message));
                    form[0].reset();
                    timeSelect.prop('disabled', true);
                } else {
                    displayMessage('error', escapeHtml(response.data.message));
                    if (response.data.retry_after) {
                        startRetryCountdown(response.data.retry_after);
                    }
                }
            },
            error: function() {
                displayMessage('error', escapeHtml(ctrlbsAjax.i18n.errorSubmitting));
            },
            complete: function() {
                if (!submitButton.prop('disabled')) {
                    submitButton.prop('disabled', false);
                }
            }
        });
    });

    // Event listeners
    dateInput.on('change', function() {
        if (validateDate()) {
            loadTimeSlots();
        }
    });

    serviceSelect.on('change', loadTimeSlots);

    // Initialize
    loadServices();
    setMaxDate();
});