<?php
/**
 * Plugin Name: Ctrl Booking For Elementor
 * Plugin URI: https://ctrldigital.tech/booking-system
 * Description: Booking system with configurable time slots, SMTP email functionality, and booking management. Integrates seamlessly with Elementor page builder.
 * Version: 1.0.0
 * Requires at least: 6.2
 * Requires PHP: 7.4
 * Author: Ctrl Digital Tech
 * Author URI: https://ctrldigital.tech
 * License: GPL v2 or later
 * License URI: https://www.gnu.org/licenses/gpl-2.0.html
 *
 * @package CtrlBookingSystem
 */

if (!defined('ABSPATH')) {
    exit;
}

define('CTRLBS_VERSION', '1.0.0');
define('CTRLBS_PLUGIN_DIR', plugin_dir_path(__FILE__));
define('CTRLBS_PLUGIN_URL', esc_url(plugin_dir_url(__FILE__)));

// Required plugin files
require_once CTRLBS_PLUGIN_DIR . 'includes/security.php';
require_once CTRLBS_PLUGIN_DIR . 'includes/init.php';
require_once CTRLBS_PLUGIN_DIR . 'includes/admin.php';
require_once CTRLBS_PLUGIN_DIR . 'includes/services.php';
require_once CTRLBS_PLUGIN_DIR . 'includes/bookings.php';
require_once CTRLBS_PLUGIN_DIR . 'includes/settings.php';
require_once CTRLBS_PLUGIN_DIR . 'includes/email.php';
require_once CTRLBS_PLUGIN_DIR . 'includes/shortcodes.php';
require_once CTRLBS_PLUGIN_DIR . 'includes/ajax-handlers.php';
require_once CTRLBS_PLUGIN_DIR . 'includes/time-slots.php';
require_once CTRLBS_PLUGIN_DIR . 'includes/template-functions.php';

/**
 * Enqueue admin scripts and styles
 */
function ctrlbs_enqueue_admin_scripts($hook): void {
    if (strpos($hook, 'ctrlbs-system') === false) {
        return;
    }

    wp_register_style(
        'ctrlbs-admin',
        CTRLBS_PLUGIN_URL . 'assets/css/admin.css',
        [],
        CTRLBS_VERSION
    );
    wp_enqueue_style('ctrlbs-admin');

    if (strpos($hook, 'ctrlbs-system-stats') !== false) {
        wp_register_style(
            'ctrlbs-stats',
            CTRLBS_PLUGIN_URL . 'assets/css/stats.css',
            [],
            CTRLBS_VERSION
        );
        wp_enqueue_style('ctrlbs-stats');
    }

    wp_register_script(
        'ctrlbs-admin',
        CTRLBS_PLUGIN_URL . 'assets/js/admin-script.js',
        ['jquery'],
        CTRLBS_VERSION,
        true
    );

    wp_localize_script('ctrlbs-admin', 'ctrlbsNonce', [
        'nonce' => wp_create_nonce('ctrlbs_admin_nonce')
    ]);

    wp_enqueue_script('ctrlbs-admin');
}
add_action('admin_enqueue_scripts', 'ctrlbs_enqueue_admin_scripts');

/**
 * Enqueue frontend scripts and styles
 */
function ctrlbs_enqueue_frontend_scripts(): void {
    if (!is_admin()) {
        wp_register_style(
            'ctrlbs',
            CTRLBS_PLUGIN_URL . 'assets/css/style.css',
            [],
            CTRLBS_VERSION
        );
        wp_enqueue_style('ctrlbs');

        wp_register_script(
            'ctrlbs-script',
            CTRLBS_PLUGIN_URL . 'assets/js/script.js',
            ['jquery'],
            CTRLBS_VERSION,
            true
        );

        $settings = get_option('ctrlbs_settings', []);

        wp_localize_script('ctrlbs-script', 'ctrlbsAjax', [
            'ajaxurl' => admin_url('admin-ajax.php'),
            'nonce' => wp_create_nonce('ctrlbs_nonce'),
            'maxAdvanceDays' => absint($settings['max_advance_days'] ?? 30),
            'businessDays' => $settings['business_days'] ?? [
                'sunday' => false,
                'monday' => true,
                'tuesday' => true,
                'wednesday' => true,
                'thursday' => true,
                'friday' => true,
                'saturday' => false
            ],
            'i18n' => [
                'selectService' => esc_html__('Select a service', 'ctrl-booking-system'),
                'errorLoading' => esc_html__('Error loading services', 'ctrl-booking-system'),
                'errorSubmitting' => esc_html__('Error submitting booking', 'ctrl-booking-system'),
                'selectDate' => esc_html__('Select a date', 'ctrl-booking-system'),
                'selectTime' => esc_html__('Select a time', 'ctrl-booking-system'),
                'closedDay' => esc_html__('Sorry, we are closed on this day', 'ctrl-booking-system'),
                'errorMessage' => esc_html__('An error occurred. Please try again.', 'ctrl-booking-system'),
                'successMessage' => esc_html__('Booking submitted successfully!', 'ctrl-booking-system')
            ]
        ]);

        wp_enqueue_script('ctrlbs-script');
    }
}
add_action('wp_enqueue_scripts', 'ctrlbs_enqueue_frontend_scripts');

/**
 * Enqueue Elementor specific styles
 */
function ctrlbs_enqueue_elementor_styles(): void {
    wp_register_style(
        'ctrlbs-elementor',
        CTRLBS_PLUGIN_URL . 'assets/css/elementor-widget.css',
        [],
        CTRLBS_VERSION
    );
    wp_enqueue_style('ctrlbs-elementor');
}
add_action('elementor/frontend/after_enqueue_styles', 'ctrlbs_enqueue_elementor_styles');

/**
 * Initialize plugin
 */
function ctrlbs_init(): void {
    add_shortcode('ctrlbs_booking_form', 'ctrlbs_booking_form_shortcode');
}
add_action('init', 'ctrlbs_init');

/**
 * Register Elementor widget
 */
function ctrlbs_register_elementor_widget($widgets_manager): void {
    if (!did_action('elementor/loaded')) {
        return;
    }
    require_once CTRLBS_PLUGIN_DIR . 'includes/elementor-widget.php';
    $widgets_manager->register(new CTRLBS_Elementor_Widget());
}
add_action('elementor/widgets/register', 'ctrlbs_register_elementor_widget');

/**
 * Add settings link to plugins page
 */
function ctrlbs_add_action_links($links): array {
    $plugin_links = [
        sprintf(
            '<a href="%s">%s</a>',
            esc_url(admin_url('admin.php?page=ctrlbs-system-settings')),
            esc_html__('Settings', 'ctrl-booking-system')
        )
    ];
    return array_merge($plugin_links, $links);
}
add_filter('plugin_action_links_' . plugin_basename(__FILE__), 'ctrlbs_add_action_links');

register_activation_hook(__FILE__, 'ctrlbs_initialize_plugin');
register_deactivation_hook(__FILE__, 'ctrlbs_deactivate_plugin');

function ctrlbs_deactivate_plugin(): void {
    wp_clear_scheduled_hook('ctrlbs_daily_cleanup');
    delete_transient('ctrlbs_cache');
    delete_option('ctrlbs_donation_notice_shown');
    delete_option('ctrlbs_donation_notice_dismissed');
}