<?php
declare(strict_types=1);

if (!defined('ABSPATH')) {
    exit;
}

if (!class_exists('CTRLBS_Security')) {
    require_once dirname(__FILE__) . '/security.php';
}

$security = CTRLBS_Security::get_instance();

require_once dirname(__FILE__) . '/services.php';
require_once dirname(__FILE__) . '/bookings.php';

/**
 * Get All Services via AJAX - Public Access
 */
function ctrlbs_get_services_ajax(): void {
    try {
        $services = ctrlbs_get_all_services();
        
        if (empty($services)) {
            wp_send_json_success(['services' => []]);
            return;
        }

        $formatted_services = array_map(function($service) {
            return [
                'id' => absint($service->id),
                'name' => esc_html($service->name),
                'price' => esc_html(ctrlbs_format_price($service->price))
            ];
        }, $services);

        wp_send_json_success(['services' => $formatted_services]);

    } catch (Exception $e) {
        wp_send_json_error(['message' => esc_html__('Error loading services', 'ctrl-booking-system')]);
    }
}

add_action('wp_ajax_get_services', 'ctrlbs_get_services_ajax');
add_action('wp_ajax_nopriv_get_services', 'ctrlbs_get_services_ajax');

/**
 * Handle Admin Actions
 */
function ctrlbs_handle_admin_actions(): void {
    try {
        $security = CTRLBS_Security::get_instance();
        check_ajax_referer('ctrlbs_admin_nonce', 'booking_nonce');

        if (!current_user_can('manage_options')) {
            throw new Exception(esc_html__('Unauthorized access', 'ctrl-booking-system'));
        }

        $action = isset($_POST['admin_action']) ? sanitize_text_field(wp_unslash($_POST['admin_action'])) : '';

        switch ($action) {
            case 'add_service':
                $name = isset($_POST['service_name']) ? sanitize_text_field(wp_unslash($_POST['service_name'])) : '';
                $price = isset($_POST['service_price']) ? floatval($_POST['service_price']) : 0;
                
                if (empty($name)) {
                    throw new Exception(esc_html__('Missing required fields', 'ctrl-booking-system'));
                }
            
                if (!$security->validate_input_length($name, 2, 100)) {
                    throw new Exception(esc_html__('Service name must be between 2 and 100 characters', 'ctrl-booking-system'));
                }

                $service_id = ctrlbs_add_service($name, $price);
                if ($service_id) {
                    wp_send_json_success([
                        'message' => esc_html__('Service added successfully', 'ctrl-booking-system'),
                        'service_id' => absint($service_id)
                    ]);
                }
                throw new Exception(esc_html__('Failed to add service', 'ctrl-booking-system'));

            case 'update_service':
                $service_id = isset($_POST['service_id']) ? absint($_POST['service_id']) : 0;
                $name = isset($_POST['service_name']) ? sanitize_text_field(wp_unslash($_POST['service_name'])) : '';
                $price = isset($_POST['service_price']) ? floatval($_POST['service_price']) : 0;

                if (empty($service_id) || empty($name)) {
                    throw new Exception(esc_html__('Missing required fields', 'ctrl-booking-system'));
                }

                if (!$security->validate_input_length($name, 2, 100)) {
                    throw new Exception(esc_html__('Service name must be between 2 and 100 characters', 'ctrl-booking-system'));
                }

                if (ctrlbs_update_service($service_id, $name, $price)) {
                    wp_send_json_success(['message' => esc_html__('Service updated successfully', 'ctrl-booking-system')]);
                }
                throw new Exception(esc_html__('Failed to update service', 'ctrl-booking-system'));

            case 'delete_service':
                $service_id = isset($_POST['service_id']) ? absint($_POST['service_id']) : 0;
                
                if (empty($service_id)) {
                    throw new Exception(esc_html__('Missing service ID', 'ctrl-booking-system'));
                }

                if (ctrlbs_delete_service($service_id)) {
                    wp_send_json_success(['message' => esc_html__('Service deleted successfully', 'ctrl-booking-system')]);
                }
                throw new Exception(esc_html__('Failed to delete service', 'ctrl-booking-system'));

            case 'cancel_booking':
                $booking_id = isset($_POST['booking_id']) ? absint($_POST['booking_id']) : 0;
                
                if (empty($booking_id)) {
                    throw new Exception(esc_html__('Missing booking ID', 'ctrl-booking-system'));
                }

                if (ctrlbs_delete_booking($booking_id)) {
                    wp_send_json_success(['message' => esc_html__('Booking cancelled successfully', 'ctrl-booking-system')]);
                }
                throw new Exception(esc_html__('Failed to cancel booking', 'ctrl-booking-system'));

            default:
                throw new Exception(esc_html__('Invalid action', 'ctrl-booking-system'));
        }
    } catch (Exception $e) {
        wp_send_json_error(['message' => esc_html($e->getMessage())]);
    }
}
add_action('wp_ajax_ctrlbs_admin_action', 'ctrlbs_handle_admin_actions');

/**
 * Handle donation notice dismissal
 */
add_action('wp_ajax_dismiss_donation_notice', function() {
    check_ajax_referer('ctrlbs_admin_nonce', 'booking_nonce');
    update_option('ctrlbs_donation_notice_shown', time());
    wp_send_json_success();
});

/**
 * Get time slots
 */
function ctrlbs_get_time_slots_ajax(): void {
    try {
        $security = CTRLBS_Security::get_instance();
        check_ajax_referer('ctrlbs_nonce', 'booking_nonce');

        $date = isset($_POST['date']) ? sanitize_text_field(wp_unslash($_POST['date'])) : '';
        $service_id = isset($_POST['service_id']) ? absint($_POST['service_id']) : 0;

        if (empty($date) || empty($service_id)) {
            throw new Exception(esc_html__('Missing required parameters', 'ctrl-booking-system'));
        }

        if (!$security->validate_date($date)) {
            throw new Exception(esc_html__('Invalid date format', 'ctrl-booking-system'));
        }

        $settings = get_option('ctrlbs_settings', []);
        $max_days = absint($settings['max_advance_days'] ?? 30);
        $booking_date = strtotime($date);
        $max_allowed_date = strtotime("+{$max_days} days");

        if ($booking_date > $max_allowed_date) {
            throw new Exception(sprintf(
                /* translators: %d: maximum number of days allowed for advance booking */
                esc_html__('Cannot book more than %d days in advance', 'ctrl-booking-system'),
                $max_days
            ));
        }

        $available_slots = ctrlbs_get_available_slots($date, $service_id);
        wp_send_json_success(['slots' => array_map('esc_html', $available_slots)]);

    } catch (Exception $e) {
        wp_send_json_error(['message' => esc_html($e->getMessage())]);
    }
}
add_action('wp_ajax_get_time_slots', 'ctrlbs_get_time_slots_ajax');
add_action('wp_ajax_nopriv_get_time_slots', 'ctrlbs_get_time_slots_ajax');

/**
 * Handle booking submissions
 */
function ctrlbs_handle_booking_ajax(): void {
    try {
        $security = CTRLBS_Security::get_instance();
        check_ajax_referer('ctrlbs_nonce', 'booking_nonce');

        // Rate limiting check
        $rate_check = $security->check_rate_limit('booking_submit');
        if (!$rate_check['allowed']) {
            wp_send_json_error([
                'message' => $rate_check['message'],
                'retry_after' => $rate_check['retry_after']
            ]);
            return;
        }

        $booking_data = [
            'name' => isset($_POST['name']) ? sanitize_text_field(wp_unslash($_POST['name'])) : '',
            'email' => isset($_POST['email']) ? sanitize_email(wp_unslash($_POST['email'])) : '',
            'phone' => isset($_POST['phone']) ? sanitize_text_field(wp_unslash($_POST['phone'])) : '',
            'date' => isset($_POST['date']) ? sanitize_text_field(wp_unslash($_POST['date'])) : '',
            'time' => isset($_POST['time']) ? sanitize_text_field(wp_unslash($_POST['time'])) : '',
            'service_id' => isset($_POST['service_id']) ? absint($_POST['service_id']) : 0
        ];

        foreach ($booking_data as $key => $value) {
            if (empty($value)) {
                throw new Exception(esc_html__('Please fill all required fields', 'ctrl-booking-system'));
            }
        }

        if (!$security->validate_input_length($booking_data['name'], 2, 50) ||
            !$security->validate_input_length($booking_data['phone'], 7, 20) ||
            !$security->validate_input_length($booking_data['email'], 5, 100)) {
            throw new Exception(esc_html__('One or more fields exceed the allowed length.', 'ctrl-booking-system'));
        }

        $booking_id = ctrlbs_create_booking($booking_data);
        $email_sent = ctrlbs_send_confirmation_email($booking_data, $booking_id);
        
        wp_send_json_success([
            'message' => esc_html__('Booking created successfully', 'ctrl-booking-system'),
            'booking_id' => absint($booking_id),
            'email_sent' => (bool)$email_sent
        ]);

    } catch (Exception $e) {
        wp_send_json_error(['message' => esc_html($e->getMessage())]);
    }
}
add_action('wp_ajax_ctrlbs_booking_submit', 'ctrlbs_handle_booking_ajax');
add_action('wp_ajax_nopriv_ctrlbs_booking_submit', 'ctrlbs_handle_booking_ajax');