<?php
declare(strict_types=1);

if (!defined('ABSPATH')) {
    exit;
}

class CTRLBS_Error {
    private static $error_messages = [
        'db_error' => 'A database error occurred. Please try again later.',
        'invalid_input' => 'Please check your input and try again.',
        'unauthorized' => 'You are not authorized to perform this action.',
        'service_error' => 'Unable to process service request.',
        'booking_error' => 'Unable to process booking request.',
        'validation_error' => 'Please ensure all fields are filled correctly.',
        'security_error' => 'Security verification failed.',
        'encryption_error' => 'Unable to process secure data.',
        'rate_limit' => 'Too many attempts. Please try again in %d minutes.',
        'rate_limit_remaining' => 'You have %d attempts remaining.',
    ];

    public static function get_user_message(string $error_type): string {
        return self::$error_messages[$error_type] ?? 'An unexpected error occurred.';
    }
}

class CTRLBS_Security {
    private const CIPHER_METHOD = 'aes-256-cbc';
    private static $instance = null;
    private static $encryption_key;

    private function __construct() {
        // Private constructor to prevent direct instantiation
    }

    public static function get_instance() {
        if (null === self::$instance) {
            self::$instance = new self();
        }
        return self::$instance;
    }

    private function get_encryption_key() {
        if (!self::$encryption_key) {
            if (!defined('AUTH_KEY') || !defined('SECURE_AUTH_KEY') ||
                empty(AUTH_KEY) || empty(SECURE_AUTH_KEY)) {

                global $wpdb;
                $site_url = get_site_url();
                $db_prefix = $wpdb->prefix;
                self::$encryption_key = substr(hash('sha256', $site_url . $db_prefix), 0, 32);

                add_action('admin_notices', function() {
                    echo '<div class="error"><p>';
                    echo 'Warning: WordPress salt keys not configured. Please update wp-config.php with proper salt 
keys for 
better security.';
                    echo '</p></div>';
                });
            } else {
                self::$encryption_key = substr(hash('sha256', AUTH_KEY . SECURE_AUTH_KEY), 0, 32);
            }
        }
        return self::$encryption_key;
    }

    public function encrypt_value($value) {
        if (empty($value)) return '';

        try {
            $encryption_key = $this->get_encryption_key();
            $iv = openssl_random_pseudo_bytes(openssl_cipher_iv_length(self::CIPHER_METHOD));
            $encrypted = openssl_encrypt($value, self::CIPHER_METHOD, $encryption_key, 0, $iv);

            if ($encrypted === false) {
                CTRLBS_Error::log_error('encryption_error', 'Failed to encrypt value');
                return false;
            }

            return base64_encode($iv . $encrypted);
        } catch (Exception $e) {
            CTRLBS_Error::log_error('encryption_error', $e->getMessage());
            return false;
        }
    }

    public function decrypt_value($encrypted_value) {
        if (empty($encrypted_value)) return '';

        try {
            $encryption_key = $this->get_encryption_key();
            $data = base64_decode($encrypted_value);
            $iv_length = openssl_cipher_iv_length(self::CIPHER_METHOD);

            $iv = substr($data, 0, $iv_length);
            $encrypted = substr($data, $iv_length);

            $decrypted = openssl_decrypt($encrypted, self::CIPHER_METHOD, $encryption_key, 0, $iv);

            if ($decrypted === false) {
                CTRLBS_Error::log_error('encryption_error', 'Failed to decrypt value');
                return false;
            }

            return $decrypted;
        } catch (Exception $e) {
            CTRLBS_Error::log_error('encryption_error', $e->getMessage());
            return false;
        }
    }



    public function check_rate_limit(string $key, int $max = 3, int $window = 300): array {
        $attempts = (int) get_transient('ctrlbs_rate_limit_' . $key);
        
        if ($attempts >= $max) {
            $time_remaining = (int) get_transient('ctrlbs_rate_limit_time_' . $key);
            return [
                'allowed' => false,
                'message' => __('Too many booking attempts. Please try again later.', 'ctrl-booking-system'),
                'retry_after' => max(0, $time_remaining)
            ];
        }
        
        set_transient('ctrlbs_rate_limit_' . $key, $attempts + 1, $window);
        if ($attempts === 0) {
            set_transient('ctrlbs_rate_limit_time_' . $key, $window, $window);
        }
        
        return ['allowed' => true];
    }


    public function clear_rate_limit(string $key): void {
        delete_transient('ctrlbs_rate_limit_' . $key);
        delete_transient('ctrlbs_rate_limit_time_' . $key);
    }

    /**
     * Phone Number Validation.
     *
     * @param string $phone Phone number.
     * @return bool Returns true if valid, false otherwise.
     */
    public function validate_phone(string $phone): bool {
        // Allow only digits, spaces, dashes, and plus sign
        $cleaned = preg_replace('/[^\d\+\-\s]/', '', $phone);
        // Simple pattern to match phone numbers
        $pattern = '/^\+?[0-9\-\s]{7,20}$/';
        return preg_match($pattern, $cleaned) === 1;
    }

    /**
     * Email Validation.
     *
     * @param string $email Email address.
     * @return bool Returns true if valid, false otherwise.
     */
    public function validate_email(string $email): bool {
        return is_email($email) !== false;
    }

    /**
     * Date Validation.
     *
     * @param string $date Date string.
     * @return bool Returns true if valid, false otherwise.
     */
    public function validate_date(string $date): bool {
        $d = DateTime::createFromFormat('Y-m-d', $date);
        return $d && $d->format('Y-m-d') === $date;
    }

    /**
     * Sanitize input string.
     *
     * @param string $input Input string.
     * @return string Sanitized string.
     */
    public function sanitize_input(string $input): string {
        return sanitize_text_field($input);
    }

    /**
     * Sanitize request data recursively.
     *
     * @param array $data Request data.
     * @return array Sanitized data.
     */
    public function sanitize_request_data(array $data): array {
        $clean = [];
        foreach ($data as $key => $value) {
            if (is_array($value)) {
                $clean[$key] = $this->sanitize_request_data($value);
            } else {
                $clean[$key] = $this->sanitize_input($value);
            }
        }
        return $clean;
    }

    /**
     * Validate input length.
     *
     * @param string $input Input string.
     * @param int $min Minimum length.
     * @param int $max Maximum length.
     * @return bool Returns true if length is within bounds, false otherwise.
     */
    public function validate_input_length(string $input, int $min, int $max): bool {
        $length = mb_strlen($input);
        return $length >= $min && $length <= $max;
    }

    /**
     * Validate CSRF token format.
     *
     * @param string $token CSRF token.
     * @return bool Returns true if token is valid, false otherwise.
     */
    public function validate_csrf_token_format(string $token): bool {
        return !empty($token) && preg_match('/^[a-f0-9]{32}$/', $token) === 1;
    }

    /**
     * Test for SQL Injection patterns.
     *
     * @param string $input Input string.
     * @return bool Returns true if input is safe, false if potential SQL injection detected.
     */
    public function test_sql_injection(string $input): bool {
        $dangerous_patterns = [
            '/[\'"\\\\]/',
            '/;/',
            '/\b(?:SELECT|INSERT|UPDATE|DELETE|DROP|ALTER|UNION|CREATE|TRUNCATE)\b/i',
            '/\bOR\b.+?(?:=|LIKE)\b/i',
            '/\bAND\b.+?(?:=|LIKE)\b/i',
            '/--/',
            '/\/\*|\*\//',
            '/\bSYSTEM_USER\b/i',
            '/\bSCHEMA\b/i',
            '/\bINFORMATION_SCHEMA\b/i',
            '/\bSYS\b/i'
        ];

        foreach ($dangerous_patterns as $pattern) {
            if (preg_match($pattern, $input)) {
                return false; // Potential SQL injection detected
            }
        }
        return true; // Input is safe
    }

/**
 * Test for XSS prevention.
 *
 * @param string $input Input string.
 * @return bool Returns true if XSS is detected, false otherwise.
 */
public function test_xss_prevention(string $input): bool {
    // Normalize the input
    $normalized_input = strtolower(trim($input));
    $normalized_input = str_replace([' ', "\t", "\n", "\r"], '', $normalized_input);

    // 1. Check for 'javascript:' protocol
    if (strpos($normalized_input, 'javascript:') === 0) {
        return true; // XSS detected
    }

    // 2. Use WordPress's wp_kses to strip unwanted HTML tags and attributes
    $allowed_html = []; // Disallow all HTML tags
    $sanitized = wp_kses($input, $allowed_html);

    // If sanitized differs from input, there were disallowed tags or attributes
    if ($sanitized !== $input) {
        return true; // XSS detected
    }

    // 3. Additional regex checks for common XSS vectors
    // Detect event handlers like onerror, onload, onclick, etc.
    if (preg_match('/on\w+=/', $input)) {
        return true; // XSS detected
    }

    // 4. Detect script tags or any inline JavaScript
    if (preg_match('/<script\b[^>]*>(.*?)<\/script>/is', $input)) {
        return true; // XSS detected
    }

    // 5. Detect any suspicious URI schemes other than allowed ones (e.g., http, https)
    if (preg_match('/<a\s+[^>]*href=["\']?javascript:/i', $input)) {
        return true; // XSS detected
    }

    return false; // No XSS detected
}

    /**
     * Validate special characters in input.
     *
     * @param string $input Input string.
     * @return bool Returns true if no disallowed characters are present, false otherwise.
     */
    public function validate_special_chars(string $input): bool {
        return !preg_match('/[<>"\']/', $input);
    }

    /**
     * Generate a CSRF token.
     *
     * @return string CSRF token.
     */
    public function generate_csrf_token(): string {
        return bin2hex(random_bytes(16));
    }

    /**
     * Validate the origin of the request.
     *
     * @return bool Returns true if origin is allowed, false otherwise.
     */
    public function validate_request_origin(): bool {
        $origin = isset($_SERVER['HTTP_ORIGIN']) 
            ? sanitize_text_field(wp_unslash($_SERVER['HTTP_ORIGIN'])) 
            : '';
        $site_url = get_site_url();
        return empty($origin) || strpos($origin, $site_url) === 0;
    }

    /**
     * Get allowed origins for requests.
     *
     * @return array List of allowed origin URLs.
     */
    private function get_allowed_origins(): array {
        return [get_site_url()];
    }

    /**
     * Throttle requests to prevent abuse.
     *
     * @param string $key Unique key for throttling.
     * @param int $limit Maximum number of allowed requests.
     * @param int $window Time window in seconds.
     * @return bool Returns true if request is allowed, false otherwise.
     */
    public function throttle_requests(string $key, int $limit = 60, int $window = 60): bool {
        $count = (int) get_transient('ctrlbs_throttle_' . $key) ?: 0;
        if ($count >= $limit) {
            return false;
        }
        set_transient('ctrlbs_throttle_' . $key, $count + 1, $window);
        return true;
    }

    /**
     * Validate the HTTP request method.
     *
     * @param string $method HTTP method to validate against.
     * @return bool Returns true if method matches, false otherwise.
     */
    public function validate_request_method(string $method): bool {
        if (!isset($_SERVER['REQUEST_METHOD'])) {
            return false;
        }
        return $_SERVER['REQUEST_METHOD'] === strtoupper($method);
    }

    /**
     * Validate IP address format.
     *
     * @param string $ip IP address.
     * @return bool Returns true if valid, false otherwise.
     */
    public function validate_ip(string $ip): bool {
        return filter_var($ip, FILTER_VALIDATE_IP) !== false;
    }

    /**
     * Validate booking data.
     *
     * @param array $data Booking data.
     * @return bool Returns true if data is valid, false otherwise.
     */
    public function validate_booking_data(array $data): bool {
        $required = ['email', 'phone', 'date'];
        foreach ($required as $field) {
            if (!isset($data[$field]) || empty($data[$field])) {
                return false;
            }
        }

        if (!$this->validate_email($data['email'])) return false;
        if (!$this->validate_phone($data['phone'])) return false;
        if (!$this->validate_date($data['date'])) return false;

        return true;
    }

    /**
     * Sanitize SQL input using WordPress's prepare method.
     *
     * @param mixed $input Input data.
     * @return mixed Sanitized input.
     */
    public function sanitize_sql_input($input) {
        global $wpdb;
        if (is_array($input)) {
            return array_map([$this, 'sanitize_sql_input'], $input);
        }
        return $wpdb->prepare('%s', $input);
    }

        /**
     * Clear all rate limits for an IP address.
     *
     * @param string $ip IP address to clear limits for.
     */
    public function clear_all_rate_limits(string $ip): void {
        global $wpdb;
        $like = $wpdb->esc_like('ctrlbs_rate_limit_' . md5($ip)) . '%';
        $wpdb->query($wpdb->prepare(
            "DELETE FROM {$wpdb->options} WHERE option_name LIKE %s",
            $like
        ));
    }
}



