<?php
declare(strict_types=1);

if (!defined('ABSPATH')) {
    exit;
}

function ctrlbs_register_settings(): void {
    register_setting(
        'ctrlbs_settings',     // Option group
        'ctrlbs_settings',     // Option name
        [
            'type' => 'array',
            'sanitize_callback' => 'ctrlbs_sanitize_settings',
            'default' => [
                'smtp_host' => '',
                'smtp_port' => '587',
                'smtp_username' => '',
                'smtp_password' => '',
                'smtp_secure' => 'tls',
                'admin_email' => get_option('admin_email'),
                'currency' => 'USD',
                'business_hours_start' => '09:00',
                'business_hours_end' => '17:00',
                'slot_duration' => '60',
                'max_bookings_per_slot' => '1', 
                'max_advance_days' => '30',
                'business_days' => [
                    'monday' => true,
                    'tuesday' => true,
                    'wednesday' => true,
                    'thursday' => true,
                    'friday' => true,
                    'saturday' => false,
                    'sunday' => false
                ]
            ]
        ]
    );
}
add_action('admin_init', 'ctrlbs_register_settings');

function ctrlbs_sanitize_settings($input): array {
    if (!is_array($input)) {
        return [];
    }

    $sanitized = [];

    // SMTP Settings
    $sanitized['smtp_host'] = sanitize_text_field($input['smtp_host'] ?? '');
    $sanitized['smtp_port'] = absint($input['smtp_port'] ?? 587);
    $sanitized['smtp_username'] = sanitize_text_field($input['smtp_username'] ?? '');
    $sanitized['smtp_password'] = sanitize_text_field($input['smtp_password'] ?? '');
    $sanitized['smtp_secure'] = in_array($input['smtp_secure'], ['tls', 'ssl'], true) 
        ? $input['smtp_secure'] 
        : 'tls';

    // Email Settings
    $sanitized['admin_email'] = sanitize_email($input['admin_email'] ?? get_option('admin_email'));

    // Business Settings
    $sanitized['currency'] = in_array($input['currency'], ['USD', 'EUR', 'GBP'], true)
        ? $input['currency']
        : 'USD';

    // Time Settings
    $sanitized['business_hours_start'] = preg_match('/^([01]?[0-9]|2[0-3]):[0-5][0-9]$/', $input['business_hours_start'] ?? '')
        ? $input['business_hours_start']
        : '09:00';
    
    $sanitized['business_hours_end'] = preg_match('/^([01]?[0-9]|2[0-3]):[0-5][0-9]$/', $input['business_hours_end'] ?? '')
        ? $input['business_hours_end']
        : '17:00';

    $sanitized['slot_duration'] = in_array($input['slot_duration'], ['30', '60', '90', '120'], true)
        ? $input['slot_duration']
        : '60';

    $sanitized['max_bookings_per_slot'] = min(50, max(1, absint($input['max_bookings_per_slot'] ?? 1)));
    $sanitized['max_advance_days'] = min(365, max(1, absint($input['max_advance_days'] ?? 30)));

    // Business Days
    $sanitized['business_days'] = [];
    $days = ['monday', 'tuesday', 'wednesday', 'thursday', 'friday', 'saturday', 'sunday'];
    foreach ($days as $day) {
        $sanitized['business_days'][$day] = !empty($input['business_days'][$day]);
    }

    return $sanitized;
}

function ctrlbs_get_settings(): array {
    return get_option('ctrlbs_settings', []);
}

function ctrlbs_update_settings(array $settings): bool {
    $security = CTRLBS_Security::get_instance();
        
    // Encrypt sensitive data
    if (!empty($settings['smtp_password'])) {
        $settings['smtp_password'] = $security->encrypt_value($settings['smtp_password']);
    }
    if (!empty($settings['smtp_username'])) {
        $settings['smtp_username'] = $security->encrypt_value($settings['smtp_username']);
    }
    
    return update_option('ctrlbs_settings', $settings);
}

function ctrlbs_get_currency_symbol(): string {
    $settings = ctrlbs_get_settings();
    $currencies = [
        'USD' => '$',
        'EUR' => '€',
        'GBP' => '£'
    ];
    return $currencies[$settings['currency'] ?? 'USD'] ?? '$';
}
