<?php
declare(strict_types=1);

if (!defined('ABSPATH')) {
    exit;
}

/**
 * Format price with currency symbol
 */
function ctrlbs_format_price(mixed $price): string {
    try {
        if (is_numeric($price)) {
            $price_float = (float)$price;
        } else {
            $price_float = 0.00;
        }
        
        // Get currency symbol and properly decode it
        $symbol = html_entity_decode(ctrlbs_get_currency_symbol(), ENT_QUOTES | ENT_HTML5, 'UTF-8');
        
        return $symbol . ' ' . number_format($price_float, 2);
        
    } catch (Exception $e) {
        return '$0.00';
    }
}

/**
 * Format date according to WordPress settings
 */
function ctrlbs_format_date(string $date): string {
    try {
        $timestamp = strtotime($date);
        if ($timestamp === false) {
            throw new Exception('Invalid date format');
        }
        return esc_html(date_i18n(get_option('date_format'), $timestamp));
    } catch (Exception $e) {
        return esc_html($date);
    }
}

/**
 * Format time according to WordPress settings
 */
function ctrlbs_format_time(string $time): string {
    try {
        $timestamp = strtotime($time);
        if ($timestamp === false) {
            throw new Exception('Invalid time format');
        }
        return esc_html(date_i18n(get_option('time_format'), $timestamp));
    } catch (Exception $e) {
        return esc_html($time);
    }
}

/**
 * Get admin page URL for booking system
 */
function ctrlbs_get_admin_page_url(string $page = ''): string {
    $url = admin_url('admin.php?page=ctrlbs-system' . ($page ? '-' . sanitize_key($page) : ''));
    return esc_url($url);
}

/**
 * Display admin notice
 */
function ctrlbs_admin_notice(string $message, string $type = 'success'): void {
    $allowed_types = ['success', 'error', 'warning', 'info'];
    $notice_type = in_array($type, $allowed_types) ? $type : 'info';

    add_action('admin_notices', function() use ($message, $notice_type) {
        printf(
            '<div class="notice notice-%1$s is-dismissible"><p>%2$s</p></div>',
            esc_attr($notice_type),
            esc_html($message)
        );
    });
}

/**
 * Format datetime for database
 */
function ctrlbs_format_datetime_for_db(string $date, string $time): string {
    try {
        $date = sanitize_text_field($date);
        $time = sanitize_text_field($time);
        
        if (!ctrlbs_validate_date($date) || !ctrlbs_validate_time($time)) {
            throw new Exception('Invalid date or time format');
        }

        $datetime = new DateTime($date . ' ' . $time);
        return $datetime->format('Y-m-d H:i:s');
    } catch (Exception $e) {
        return current_time('mysql');
    }
}

/**
 * Validate date format
 */
function ctrlbs_validate_date(string $date): bool {
    $format = 'Y-m-d';
    $d = DateTime::createFromFormat($format, sanitize_text_field($date));
    return $d && $d->format($format) === $date;
}

/**
 * Validate time format
 */
function ctrlbs_validate_time(string $time): bool {
    $format = 'H:i';
    $t = DateTime::createFromFormat($format, sanitize_text_field($time));
    return $t && $t->format($format) === $time;
}

/**
 * Format booking status
 */
function ctrlbs_format_status(string $status): string {
    $allowed_statuses = [
        'pending' => __('Pending', 'ctrl-booking-system'),
        'confirmed' => __('Confirmed', 'ctrl-booking-system'),
        'cancelled' => __('Cancelled', 'ctrl-booking-system')
    ];

    $status = strtolower(sanitize_text_field($status));
    return esc_html($allowed_statuses[$status] ?? __('Unknown', 'ctrl-booking-system'));
}

/**
 * Format customer name
 */
function ctrlbs_format_customer_name(string $name): string {
    return esc_html(sanitize_text_field($name));
}

/**
 * Format email address
 */
function ctrlbs_format_email(string $email): string {
    return esc_html(sanitize_email($email));
}

/**
 * Format phone number
 */
function ctrlbs_format_phone(string $phone): string {
    return esc_html(sanitize_text_field($phone));
}

/**
 * Get service name by ID
 */
function ctrlbs_get_service_name(int $service_id): string {
    $service = ctrlbs_get_service(absint($service_id));
    return $service ? esc_html($service->name) : esc_html__('Unknown Service', 'ctrl-booking-system');
}

/**
 * Format boolean values for display
 */
function ctrlbs_format_boolean(bool $value): string {
    return esc_html($value ? __('Yes', 'ctrl-booking-system') : __('No', 'ctrl-booking-system'));
}

/**
 * Format error messages
 */
function ctrlbs_format_error(string $message): string {
    return sprintf(
        '<div class="error-message">%s</div>',
        esc_html($message)
    );
}

/**
 * Format success messages
 */
function ctrlbs_format_success(string $message): string {
    return sprintf(
        '<div class="success-message">%s</div>',
        esc_html($message)
    );
}