<?php
declare(strict_types=1);

if (!defined('ABSPATH')) {
    exit;
}

function ctrlbs_get_available_slots(string $date, int $service_id): array {
    try {
        // Get settings with defaults
        $settings = get_option('ctrlbs_settings', []);
        $default_settings = [
            'business_hours_start' => '09:00',
            'business_hours_end' => '17:00',
            'slot_duration' => 60,
            'max_bookings_per_slot' => 1
        ];
        $settings = array_merge($default_settings, $settings);

        // Use WordPress current_time() for proper timezone handling
        $current_date = current_time('Y-m-d');
        $current_time = current_time('H:i');

        // Validate inputs
        if (!ctrlbs_validate_date_format($date)) {
            throw new Exception('Invalid date format');
        }

        $start_timestamp = strtotime($settings['business_hours_start']);
        $end_timestamp = strtotime($settings['business_hours_end']);
        $slot_duration = (int)$settings['slot_duration'] * 60;

        if ($start_timestamp === false || $end_timestamp === false) {
            throw new Exception('Invalid time format in settings');
        }

        $booked_slots = ctrlbs_get_booked_slots($date, $service_id);
        $available_slots = [];

        for ($time = $start_timestamp; $time + $slot_duration <= $end_timestamp; $time += $slot_duration) {
            $current_slot = gmdate('H:i', $time);            
            // If date is today, skip past times
            if ($date === $current_date && $current_slot <= $current_time) {
                continue;
            }
            
            if (ctrlbs_is_slot_available_for_booking($date, $current_slot, $service_id, $settings)) {
                $available_slots[] = $current_slot;
            }
        }

        return $available_slots;

    } catch (Exception $e) {
        return [];
    }
}

function ctrlbs_get_booked_slots(string $date, int $service_id): array {
    global $wpdb;
    
    try {
        return $wpdb->get_col($wpdb->prepare(
            "SELECT DATE_FORMAT(booking_time, '%%H:%%i') 
             FROM {$wpdb->prefix}ctrlbs_bookings 
             WHERE booking_date = %s 
             AND service_id = %d",
            $date,
            $service_id
        )) ?: [];
    } catch (Exception $e) {
        return [];
    }
}

function ctrlbs_is_slot_available(string $date, string $time, int $service_id): bool {
    try {
        $settings = get_option('ctrlbs_settings', []);
        return ctrlbs_is_slot_available_for_booking($date, $time, $service_id, $settings);
    } catch (Exception $e) {
        return false;
    }
}

function ctrlbs_is_slot_available_for_booking(
    string $date,
    string $time,
    int $service_id,
    array $settings
): bool {
    try {
        // Check if slot is within business hours
        $slot_timestamp = strtotime($time);
        $start_timestamp = strtotime($settings['business_hours_start']);
        $end_timestamp = strtotime($settings['business_hours_end']);

        if ($slot_timestamp < $start_timestamp || $slot_timestamp >= $end_timestamp) {
            return false;
        }

        // If date is today, check if time has passed
        if ($date === gmdate('Y-m-d') && $time <= gmdate('H:i')) {
            return false;
        }

        // Check booking count against max allowed
        $booking_count = ctrlbs_get_slot_booking_count($date, $time, $service_id);
        $max_bookings = (int)($settings['max_bookings_per_slot'] ?? 1);

        return $booking_count < $max_bookings;

    } catch (Exception $e) {
        return false;
    }
}

function ctrlbs_get_slot_booking_count(string $date, string $time, int $service_id): int {
    global $wpdb;
    
    try {
        return (int)$wpdb->get_var($wpdb->prepare(
            "SELECT COUNT(*) 
             FROM {$wpdb->prefix}ctrlbs_bookings 
             WHERE booking_date = %s 
             AND booking_time = %s 
             AND service_id = %d",
            $date,
            $time,
            $service_id
        ));
    } catch (Exception $e) {
        return 0;
    }
}

function ctrlbs_validate_date_format(string $date): bool {
    $format = 'Y-m-d';
    $d = DateTime::createFromFormat($format, $date);
    return $d && $d->format($format) === $date;
}

function ctrlbs_is_business_day(string $date): bool {
    $settings = get_option('ctrlbs_settings', []);
    $business_days = $settings['business_days'] ?? [];
    
    // Get day of week (lowercase) from date
    $day_of_week = strtolower(gmdate('l', strtotime($date)));
    
    return !empty($business_days[$day_of_week]);
}